;*****************************************************************************
;*                                                                           *
;* RGB frame                                                                 *
;* Copyright (C) 2020 by Walter Lain <kcswalter@member.fsf.org>              *
;*                                                                           *
;* This program is free software; you can redistribute it and/or modify      *
;* it under the terms of the GNU General Public License as published by      *
;* the Free Software Foundation; either version 3 of the License, or         *
;* (at your option) any later version.                                       *
;*                                                                           *
;* This program is distributed in the hope that it will be useful,           *
;* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
;* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
;* GNU General Public License for more details.                              *
;*                                                                           *
;* You should have received a copy of the GNU General Public License         *
;* along with this program; if not, write to the Free Software               *
;* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
;*                                                                           *
;*****************************************************************************

; ________________________________________________
; |          SAF routines for RGB frame          |
; |______________________________________________|
; |   Src File                                   |
; |______________________________________________|
; |   Type      : ASM                            |
; |   Version   : 1                              |
; |   Release   : 0                              |
; |   Author    : Lain Walter                    |
; |   Date      : 10/01/21                       |
; |______________________________________________|

;_______________________________________________________SAF read sub
;Reads the program configuration bytes from the SAF
;Modified registers:
;	system
;	b_MENU_FLAGS
;	b_PWM_RED
;	b_PWM_RBK
;	b_PWM_GRN
;	b_PWM_GBK
;	b_PWM_BLU
;	b_PWM_BBK
;	b_RGB_SPD
;	b_RGB_MODE
;	b_RGB_COLOR
;	b_MENU_FLAGS
;	w_RGB_CTRL+1
;	w_RGB_CTRL
;	w_DMX_ADDR
;	b_DMX_CTRL
;Call method
;	CALL SAF_Read
SAF_Read:
	BANKSEL NVMADRL			;Bank 16
	MOVLF c_SAFADDR_H,NVMADRH	;Store MSB of address
	MOVLF c_SAFADDR_L,NVMADRL	;Store LSB of address
	BCF NVMCON1,NVMREGS		;Select program memory space
	BSF NVMCON1,RD			;Start read
	MOVF NVMDATL,w			;Get LSB of word
	BZ SAF_END			;First word is 0, SAF not init, exit
	XORLW 0xAA			;Check if SAF is written correctly
	BNZ SAF_END			; No, exit
	CLRF FSR0H			; Yes, set indirect address to point R0H
	MOVLF 0xF0,FSR0L		;
	INCF NVMADRL,f			; Increment SAF address
	MOVLF c_SAF_SIZE,R7L		; Load byte counter (I'm reading "bytes", that is LSBs of the SAF cells)
SAF_RD	BSF NVMCON1,RD			;Start read
	MOVFF NVMDATL,INDF0		;Save LSB of SAF word
	INCF FSR0L,f			;Increment RAM position
	INCF NVMADRL,f			;Increment SAF address
	DECFSZ R7L,f			;
	GOTO SAF_RD
	BANKSEL b_PWM_RED		;Bank 0
	MOVFF R0H,b_PWM_RED		;Restore RED set point
	MOVFF R0L,b_PWM_RBK		;Restore fixed RED set point
	MOVFF R1H,b_PWM_GRN		;Restore GREEN set point
	MOVFF R1L,b_PWM_GBK		;Restore fixed GREEN set point
	MOVFF R2H,b_PWM_BLU		;Restore BLUE set point
	MOVFF R2L,b_PWM_BBK		;Restore fixed BLUE set point
	MOVFF R3H,b_RGB_SPD		;Restore RGB cycle speed
	MOVFF R3L,b_RGB_MODE		;Restore RGB cycle mode
	MOVFF R4H,b_RGB_COLOR		;Restore RGB fixed color index
	MOVFF R4L,b_MENU_FLAGS		;Restore Menu flags
	MOVFF R5H,w_RGB_CTRL+1		;Restore RGB control flags
	MOVFF R5L,w_RGB_CTRL		;
	MOVFF R6H,b_DMX_CTRL		;Restore DMX control flags
	MOVFF R6L,w_DMX_ADDR+1		;Restore DMX Address
	MOVFF R7H,w_DMX_ADDR		;
SAF_END	RETURN				;End

;_______________________________________________________SAF write sub
;Writes the program configuration bytes into the SAF
; NOTE: this will freeze the MCU while executing
;Modified registers:
;	system
;	b_MENU_FLAGS
;	b_PWM_RED
;	b_PWM_RBK
;	b_PWM_GRN
;	b_PWM_GBK
;	b_PWM_BLU
;	b_PWM_BBK
;	b_RGB_SPD
;	b_RGB_MODE
;	b_RGB_COLOR
;	b_MENU_FLAGS
;	w_RGB_CTRL+1
;	w_RGB_CTRL
;	w_DMX_ADDR
;	b_DMX_CTRL
;Call method
;	CALL SAF_Write
SAF_Write:
	BTFSS f_E2_WRITE		;Check if write flag set
	RETURN				; No, nothing to do, exit
	BCF f_E2_WRITE			; Yes, clear write flag
	MOVFF b_PWM_RED,R0H		;Load RED set point
	MOVFF b_PWM_RBK,R0L		;Load fixed RED set point
	MOVFF b_PWM_GRN,R1H		;Load GREEN set point
	MOVFF b_PWM_GBK,R1L		;Load fixed GREEN set point
	MOVFF b_PWM_BLU,R2H		;Load BLUE set point
	MOVFF b_PWM_BBK,R2L		;Load fixed BLUE set point
	MOVFF b_RGB_SPD,R3H		;Load RGB cycle speed
	MOVFF b_RGB_MODE,R3L		;Load RGB cycle mode
	MOVFF b_RGB_COLOR,R4H		;Load RGB fixed color index
	MOVFF b_MENU_FLAGS,R4L		;Load Menu flags
	MOVFF w_RGB_CTRL+1,R5H		;Load RGB control flags
	MOVFF w_RGB_CTRL,R5L		;
	MOVFF b_DMX_CTRL,R6H		;Load DMX control flags
	MOVFF w_DMX_ADDR+1,R6L		;Load DMX Address
	MOVFF w_DMX_ADDR,R7H		;
	BANKSEL NVMCON1			;Bank 16
	MOVLF c_SAFADDR_H,NVMADRH	;Load upper 6 bits of erase address boundary
	MOVLF c_SAFADDR_L,NVMADRL	;Load lower 8 bits of erase address boundary
	BCF NVMCON1,NVMREGS		;Choose PFM memory area
	BSF NVMCON1,FREE		;Specify an erase operation
	BSF NVMCON1,WREN		;Set write/erase operation
	CALL SAF_UNL			;Erase SAF block
	CLRF FSR0H			;set indirect address to point R0H
	MOVLF 0xF0,FSR0L		;
	MOVLF c_SAFADDR_H,NVMADRH	;Load upper 6 bits of initial write address
	MOVLF c_SAFADDR_L,NVMADRL	;Load lower 8 bits of initial write address
	BSF NVMCON1,LWLO		;Set only load write latches
	MOVLF 0xAA,NVMDATL		;Set first cell as written (read sub will not consider data valid otherwise)
	CALL SAF_UNL			;Load latches
SAF_WR	INCF NVMADRL,f			;Increment SAF address
	MOVIW FSR0++			;
	MOVWF NVMDATL			;Load data byte
	MOVF NVMADRL,w			;Check if reached end of boundary
	XORLW c_SAFADDR_E		;
	BZ SAF_WRT			; Yes, write into memory
	CALL SAF_UNL			; No, load latches
	GOTO SAF_WR			; Repeat
SAF_UNL	CLI				;Disable interrupts
	MOVLW 0x55			;Unlock sequence
	MOVWF NVMCON2			; 1) Load 55h into NVMCON2
	MOVLW 0xAA			;
	MOVWF NVMCON2			; 2) Load AAH into NVMCON2
	BSF NVMCON1,WR			; 3) Set WR bit to begin write/erase
	SEI				;Enable interrupts
	RETURN
SAF_WRT	BCF NVMCON1,LWLO		;Latch writes complete, now write memory
	CALL SAF_UNL			;Perform required unlock sequence
	BCF NVMCON1,WREN		;Done, disable write/erase
;	BTFSS NVMCON1,WRERR		;Check if write error? For now, leave it
	RETURN
